/*
    FreeRTOS V7.2.0 - Copyright (C) 2012 Real Time Engineers Ltd.
	

    ***************************************************************************
     *                                                                       *
     *    FreeRTOS tutorial books are available in pdf and paperback.        *
     *    Complete, revised, and edited pdf reference manuals are also       *
     *    available.                                                         *
     *                                                                       *
     *    Purchasing FreeRTOS documentation will not only help you, by       *
     *    ensuring you get running as quickly as possible and with an        *
     *    in-depth knowledge of how to use FreeRTOS, it will also help       *
     *    the FreeRTOS project to continue with its mission of providing     *
     *    professional grade, cross platform, de facto standard solutions    *
     *    for microcontrollers - completely free of charge!                  *
     *                                                                       *
     *    >>> See http://www.FreeRTOS.org/Documentation for details. <<<     *
     *                                                                       *
     *    Thank you for using FreeRTOS, and thank you for your support!      *
     *                                                                       *
    ***************************************************************************


    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    >>>NOTE<<< The modification to the GPL is included to allow you to
    distribute a combined work that includes FreeRTOS without being obliged to
    provide the source code for proprietary components outside of the FreeRTOS
    kernel.  FreeRTOS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!
    
    ***************************************************************************
     *                                                                       *
     *    Having a problem?  Start by reading the FAQ "My application does   *
     *    not run, what could be wrong?                                      *
     *                                                                       *
     *    http://www.FreeRTOS.org/FAQHelp.html                               *
     *                                                                       *
    ***************************************************************************

    
    http://www.FreeRTOS.org - Documentation, training, latest information, 
    license and contact details.
    
    http://www.FreeRTOS.org/plus - A selection of FreeRTOS ecosystem products,
    including FreeRTOS+Trace - an indispensable productivity tool.

    Real Time Engineers ltd license FreeRTOS to High Integrity Systems, who sell 
    the code with commercial support, indemnification, and middleware, under 
    the OpenRTOS brand: http://www.OpenRTOS.com.  High Integrity Systems also
    provide a safety engineered and independently SIL3 certified version under 
    the SafeRTOS brand: http://www.SafeRTOS.com.
*/

/******************************************************************************
 * This project is a simple blinky style project, user controled switch, and a
 * uart1 echo server. The function creates two queues, two semaphores, six tasks.
 * It then starts the scheduler.
 *
 * Send String Uart1:
 * The Send String Uart1 is implemented by the SendStringUart1() function in
 * the myTasks.c file.  SendStringUart1() sits in a loop that causes it to
 * repeatedly block for 3000 milliseconds, before sending the string to the TX
 * send queue and setting the TX send interrupt. Once the string is sent, the
 * task loops back around to block for another 200 milliseconds.
 *
 * The Echo Server Task:
 * The Echo Server task is implemented by the RxQueueToTxQueue() function in the
 * myTasks.c file.  RxQueueToTxQueue() sits in a loop where it waits for a
 * semaphore from the uart1 RX module that '\n' has been received. it then places
 * the data on the TX queue to return to the sender.
 *
 * User switch task:
 * The User Switch Task is implemented in the vButtonCheckTask() and
 *  vButtonLED3Task() functions in the myTasks.c file.
 *
 * LED Blink Task
 * The LED Blink Task is implemented by the vLED1FlashTask and vLED2FlashTask
 * functions in the myTasks.c file. Both tasks sitin a loop that causes them to
 * repeatedly block for their set times.
 ******************************************************************************/
#include "echo.h"

/******************************************************************************/
int main( void )
{
    Params LED1, LED2, LED3;
    portTickType bttnPrd = 20;
    LED1.LED = 0x08;
    LED1.Period = 1000;
    LED2.LED = 0x01;
    LED2.Period = 150;
    LED3.LED = 0x02;
    LED3.Period = 50;

    //Prepare the hardware to run this demo.
    SetupHardware(mainCOM_TEST_BAUD_RATE, comBUFFER_LEN);

    xTaskCreate( vLEDFlashTask,    "LED1",         configMINIMAL_STACK_SIZE, &LED1,     mainLED_TASK_PRIORITY       , NULL);
    xTaskCreate( vLEDFlashTask,    "LED2",         configMINIMAL_STACK_SIZE, &LED2,     mainLED_TASK_PRIORITY       , NULL);
    xTaskCreate( vLEDFlashTask,    "LED3",         configMINIMAL_STACK_SIZE, &LED3,     mainLED_TASK_PRIORITY       , &LED3_Hndl);
    xTaskCreate( vButtonCheckTask, "SW0_IO",       configMINIMAL_STACK_SIZE, &bttnPrd,  mainButton_TASK_PRIORITY    , NULL);
    xTaskCreate( RxQueueToTxQueue, "Echo",         configMINIMAL_STACK_SIZE, NULL,      mainUSART_TASK_PRIORITY     , NULL);
    xTaskCreate( SendStringUart1,  "PrintString",  configMINIMAL_STACK_SIZE, NULL,      mainUSART_TASK_PRIORITY     , NULL);

    vTaskSuspend(LED3_Hndl);

    //start scheduler
    vTaskStartScheduler();
  
    //you should never get here
    while(1);

}//main end

/******************************************************************************/
void vApplicationMallocFailedHook( void )
{
    /* vApplicationMallocFailedHook() will only be called if
    configUSE_MALLOC_FAILED_HOOK is set to 1 in FreeRTOSConfig.h.  It is a hook
    function that will get called if a call to pvPortMalloc() fails.
    pvPortMalloc() is called internally by the kernel whenever a task, queue,
    timer or semaphore is created.  It is also called by various parts of the
    demo application.  If heap_1.c or heap_2.c are used, then the size of the
    heap available to pvPortMalloc() is defined by configTOTAL_HEAP_SIZE in
    FreeRTOSConfig.h, and the xPortGetFreeHeapSize() API function can be used
    to query the size of free heap space that remains (although it does not
    provide information on how the remaining heap might be fragmented). */
    taskDISABLE_INTERRUPTS();
    while(1);
}

/******************************************************************************/
void vApplicationIdleHook( void )
{
    /* vApplicationIdleHook() will only be called if configUSE_IDLE_HOOK is set
    to 1 in FreeRTOSConfig.h.  It will be called on each iteration of the idle
    task.  It is essential that code added to this hook function never attempts
    to block in any way (for example, call xQueueReceive() with a block time
    specified, or call vTaskDelay()).  If the application makes use of the
    vTaskDelete() API function (as this demo application does) then it is also
    important that vApplicationIdleHook() is permitted to return to its calling
    function, because it is the responsibility of the idle task to clean up
    memory allocated by the kernel to any task that has since been deleted. */
}

/*******************************************************************************
 * Run time task stack overflow checking is performed if 
 * configCHECK_FOR_STACK_OVERFLOW is defined to 1 or 2.  This hook function is
 * called if a task stack overflow is detected.  Note the system/interrupt
 * stack is not checked.
 ******************************************************************************/
void vApplicationStackOverflowHook( xTaskHandle pxTask, signed char *pcTaskName )
{
    LATFSET = 0x000B;
    
    taskDISABLE_INTERRUPTS();

    while(1);
}

/******************************************************************************/
void vApplicationTickHook( void )
{
    /* This function will be called by each tick interrupt if
    configUSE_TICK_HOOK is set to 1 in FreeRTOSConfig.h.  User code can be
    added here, but the tick hook is called from an interrupt context, so
    code must not attempt to block, and only the interrupt safe FreeRTOS API
    functions can be used (those that end in FromISR()). */
}

/******************************************************************************/
void _general_exception_handler( unsigned long ulCause, unsigned long ulStatus )
{
    /* This overrides the definition provided by the kernel.  Other exceptions
    should be handled here. */
    while(1);
}

void vAssertCalled( const char * pcFile, unsigned long ulLine )
{
volatile unsigned long ul = 0;

	( void ) pcFile;
	( void ) ulLine;

	__asm volatile( "di" );
	{
		/* Set ul to a non-zero value using the debugger to step out of this
		function. */
		while( ul == 0 )
		{
			portNOP();
		}
	}
	__asm volatile( "ei" );
}